@@ -139,6 +139,11 @@ sub generate_doc {
 	}
     } 
     
+    if ( $ds->debug )
+    {
+	print STDERR "---------------\nDocumentation '$path'\n";
+    }
+    
     # Record this request's URL base so that we have it in order to generate
     # documentation if necessary.
     
@@ -150,54 +155,55 @@ sub generate_doc {
     {
 	my $doc_string = $ds->render_doc($doc_template, $doc_defs, $doc_header, $doc_footer, $vars);
 	
-	# If POD format was requested, return the documentation as is.
+	my $url_formatter = sub {
+	    if ( $_[0] =~ qr{ ^ (node|op|path) (abs|rel|site)? [:] ( [^#?]* ) (?: [?] ( [^#]* ) )? (?: [#] (.*) )? }xs )
+	    {
+		my $arg = $1;
+		my $type = $2 || 'site';
+		my $path = $3 || '/';
+		my $params = $4;
+		my $frag = $5;
+		my $format;
+		
+		if ( $arg ne 'path' && $path =~ qr{ (.*) [.] ([^.]+) $ }x )
+		{
+		    $path = $1; $format = $2;
+		}
+		
+		return $request->generate_url({ $arg => $path, type => $type, format => $format,
+						params => $params, fragment => $frag });
+	    }
+	    else
+	    {
+		return $_[0];
+	    }
+	};
+	
+	# If Pod format was requested, return the documentation as is.  The
+	# only change we need to make is to convert our special link syntax to
+	# standard Pod syntax.
 	
 	if ( defined $format && $format eq 'pod' )
 	{
 	    $ds->_set_content_type($request, 'text/plain');
-	    return $doc_string;
+	    return $ds->convert_pod_links($doc_string, $url_formatter);
 	}
 	
 	# Otherwise, convert the POD to HTML using the PodParser and return the result.
 	
 	else
 	{
-	    my $parser = Web::DataService::PodParser->new();
-	    
-	    $parser->parse_pod($doc_string);
-	    
-	    my $url_generator = sub {
-		if ( $_[0] =~ qr{ ^ (node|op|path) (abs|rel|site)? [:] ( [^#?]* ) (?: [?] ( [^#]* ) )? (?: [#] (.*) )? }xs )
-		{
-		    my $arg = $1;
-		    my $type = $2 || 'site';
-		    my $path = $3 || '/';
-		    my $params = $4;
-		    my $frag = $5;
-		    my $format;
-		    
-		    if ( $arg ne 'path' && $path =~ qr{ (.*) [.] ([^.]+) $ }x )
-		    {
-			$path = $1; $format = $2;
-		    }
-		    
-		    return $request->generate_url({ $arg => $path, type => $type, format => $format, 
-						    params => $params, fragment => $frag });
-		}
-		else
-		{
-		    return $_[0];
-		}
-	    };
-	    
 	    my $stylesheet = $ds->node_attr($path, 'doc_stylesheet') || 
 		$ds->generate_site_url({ path => 'css/dsdoc.css' });
 	    
-	    my $doc_html = $parser->generate_html({ css => $stylesheet, tables => 1,
-						    url_generator => $url_generator });
+	    my $parser = Web::DataService::PodParser->new({ target => 'html', css => $stylesheet, 
+							    url_formatter => $url_formatter,
+							    page_title => $doc_title });
+	    
+	    $parser->parse_string_document($doc_string);
 	    
 	    $ds->_set_content_type($request, 'text/html');
-	    return $doc_html;
+	    return $parser->output;
 	}
     }
     
@@ -210,6 +216,96 @@ sub generate_doc {
 }
 
 
+# check_for_template ( path )
+# 
+# Return true if a documentation template exists for the specified node path.
+# Return false if not.  Throw an exception if the file exists but is not
+# readable. 
+
+sub check_for_template {
+
+    my ($ds, $path) = @_;
+    
+    my $doc_suffix = $ds->{template_suffix} // "";
+    
+    my $check1 = $path . '_doc' . $doc_suffix;
+    
+    return $check1 if $ds->check_doc( $check1 );
+    
+    my $check2 = $path . '/index' . $doc_suffix;
+    
+    return $check2 if $ds->check_doc( $check2 );
+    
+    return; # otherwise
+}
+
+
+# make_doc_node ( path, doc_path )
+# 
+# Create a documentation node for the specified path, reading the title from
+# the template file.  The second method parameter is the actual (relative)
+# path of the file on disk.
+
+sub make_doc_node {
+    
+    my ($ds, $path, $doc_path) = @_;
+    
+    my $new_attrs = { path => $path, title => 'NULL' };
+    
+    my $partial_contents = $ds->read_doc_partial($doc_path);
+    
+    while ( $partial_contents =~ m{ ^ =for \s+ wds_node \s* (.*) $ }gxmi )
+    {
+	my $expr = $1;
+	
+	while ( $expr )
+	{
+	    if ( $expr =~ qr{ ^ (\w+) \s* = \s* " ( (?: [^"] | \\{2} | \\" )+ ) " \s* (.*) }xs )
+	    {
+		$expr = $3;
+		my $attr = $1;
+		my $value = $2;
+		$value =~ s{\\{2}}{\\}g;
+		
+		unless ( $Web::DataService::Node::NODE_DEF{$attr} )
+		{
+		    die "500 Invalid attribute '$attr' for wds_node\n";
+		}
+		
+		$new_attrs->{$attr} = $value;
+	    }
+	    
+	    elsif ( $expr =~ qr{ ^ (\w+) \s* = \s* ( (?: [^;] | \\{2} | \\; )+ ) \s* (.*) }xs )
+	    {
+		$expr = $3;
+		my $attr = $1;
+		my $value = $2;
+		$value =~ s{\\{2}}{\\}g;
+		
+		unless ( $Web::DataService::Node::NODE_DEF{$attr} )
+		{
+		    die "500 Invalid attribute '$attr' for wds_node\n";
+		}
+		
+		$new_attrs->{$attr} = $value;
+	    }
+	    
+	    elsif ( $expr =~ qr{ ^ ; \s* (.*) }xs )
+	    {
+		$expr = $1;
+	    }
+	    
+	    else
+	    {
+		die "500 Invalid syntax for wds_node: '$expr'\n";
+	    }
+	}
+    }
+    
+    $ds->_create_path_node($new_attrs, '', '');
+}
+
+
 # get_nodelist ( )
 # 
 # Return a list of sub-nodes of the current one.  This will include all
@@ -374,4 +470,35 @@ sub _make_usage_doc {
 }
 
 
+# convert_pod_links ( doc_string )
+# 
+# Convert the contents of all L<...>, L<<...>>, etc. elements to proper links.
+
+sub convert_pod_links {
+    
+    my ($ds, $doc_string, $urlgen) = @_;
+    
+    $doc_string =~ s{L<<<(.*?)>>>}{'L<<<' . $ds->convert_pod_link($1, $urlgen) . '>>>'}ge;
+    $doc_string =~ s{L<<(.*?)>>}{'L<<' . $ds->convert_pod_link($1, $urlgen) . '>>'}ge;
+    $doc_string =~ s{L<(.*?)>}{'L<' . $ds->convert_pod_link($1, $urlgen). '>'}ge;
+    
+    return $doc_string;
+}
+
+
+sub convert_pod_link {
+    
+    my ($ds, $target, $urlgen) = @_;
+    
+    if ( $target =~ qr{ ^ (.*?) \| (.*) }xs )
+    {
+	return "$1|" . $urlgen->($2);
+    }
+    
+    else
+    {
+	return $urlgen->($target);
+    }
+}
+
 1;
