@@ -10,6 +10,8 @@ package Web::DataService::IRequest;
 
 use Carp 'croak';
 use Scalar::Util 'reftype';
+use JSON 'decode_json';
+use Try::Tiny;
 
 use Moo::Role;
 
@@ -193,6 +195,22 @@ sub filter_hash {
 }
 
 
+# param_keys ( )
+# 
+# Return a list of strings representing the cleaned parameter keys from this
+# request.  These will often be the same as the original parameter names, but
+# may be different if 'alias' or 'key' was specified in any of the relevant
+# validation rules.
+
+sub param_keys {
+    
+    my ($request) = @_;
+    
+    return $request->{valid}->keys() if $request->{valid};
+    return;
+}
+
+
 # clean_param ( name )
 # 
 # Return the cleaned value of the named parameter, or the empty string if it
@@ -264,7 +282,148 @@ sub param_given {
     my ($request, $name) = @_;
     
     return unless ref $request->{valid};
-    return exists $request->{valid}{raw}{$name};
+    return exists $request->{valid}{clean}{$name};
+}
+
+
+# validate_params ( ruleset, params )
+# 
+# Pass the given parameters to the validator, to be validated by the specified ruleset.
+# Return the validation result object.
+
+sub validate_params {
+    
+    my ($request, $rs_name, @params) = @_;
+    
+    my $context = { ds => $request->{ds}, request => $request };
+    my $result = $request->{ds}{validator}->check_params($rs_name, $context, @params);
+    
+    return $result;
+}
+
+
+# raw_body ( )
+# 
+# Return the request body as an un-decoded string. If there is none, return the empty string.
+
+sub raw_body {
+    
+    my ($request) = @_;
+    
+    return $request->{ds}{backend_plugin}->get_request_body() // '';
+}
+
+
+# decode_body ( )
+# 
+# Determine what format the request body is in, and decode it.
+
+sub decode_body {
+    
+    my ($request, $section) = @_;
+    
+    # First grab (and cache) the undecoded request body.
+    
+    unless ( defined $request->{raw_body} )
+    {
+	$request->{raw_body} = $request->{ds}{backend_plugin}->get_request_body() // '';
+    }
+    
+    # If this is empty, return the undefined value.
+    
+    return undef unless defined $request->{raw_body} && $request->{raw_body} ne '';
+    
+    # Get the submitted content type.
+    
+    my $content_type = $request->{ds}{backend_plugin}->get_content_type() // '';
+    
+    # If the body starts and ends with '{' or '[', assume the format is JSON regardless of content type.
+    
+    if ( $request->{raw_body} =~ / ^ [{] .* [}] $ | ^ [\[] .* [\]] $ /xsi )
+    {
+	try {
+	    unless ( defined $request->{decoded_body} )
+	    {
+		# print STDERR "About to decode\n";
+		$request->{decoded_body} = JSON->new->utf8->relaxed->decode($request->{raw_body});
+		# print STDERR "Decoded: " . $request->{decoded_body} . "\n";
+	    }
+	}
+	
+	catch {
+	    $request->{decoded_body} = '';
+	    $request->{body_error} = $_;
+	    $request->{body_error} =~ s{ at /.*}{};
+	    # print STDERR "Error: $request->{body_error}\n";
+	};
+	
+	return ($request->{decoded_body}, $request->{body_error});
+    }
+    
+    # If the content type is application/x-www-form-urlencoded, then unpack it as a single record.
+
+    elsif ( $content_type =~ qr{^application/x-www-form-urlencoded}xsi )
+    {
+	my @chunks = split(/&/, $request->{raw_body});
+	$request->{decoded_body} = { };
+	
+	foreach my $chunk ( @chunks )
+	{
+	    my ($var, $value) = split(/=/, $chunk, 2);
+
+	    $var =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
+	    $value =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
+
+	    $request->{decoded_body}{$var} = $value;
+	}
+	
+	return $request->{decoded_body};
+    }
+    
+    # Otherwise, split into rows and return.
+    
+    else
+    {
+	my @lines = split(/[\r\n]+/, $request->{raw_body});
+	$request->{decoded_body} = \@lines;
+	
+	return $request->{decoded_body};
+    }
+}
+
+
+# exception ( code, message )
+# 
+# Return an exception object with the specified HTTP result code and
+# message. This can be used to return an error result.
+
+sub exception {
+    
+    my ($request, $code, $message) = @_;
+    
+    croak "Bad exception code '$code', must be an HTTP result code"
+	unless defined $code && $code =~ qr{^\d\d\d$};
+    
+    unless ( $message )
+    {
+	if ( $code eq '400' )
+	{
+	    $message = 'Parameter error';
+	}
+	
+	elsif ( $code eq '404' )
+	{
+	    $message = 'Not found';
+	}
+	
+	else
+	{
+	    $message = 'Internal error: please contact the website administrator';
+	}
+    }
+    
+    my $exception = { code => $code, message => $message };
+    return bless $exception, 'Web::DataService::Exception';
 }
 
 
@@ -314,15 +473,25 @@ sub debug {
     
     my ($request) = @_;
     
-    return $request->{ds}->debug;
+    return $Web::DataService::DEBUG;
 }
 
 
-# process_record ( record, steps )
+# debug_line ( )
+#
+# Output the specified line(s) of text for debugging purposes.
+
+sub debug_line {
+
+    print STDERR "$_[1]\n" if $Web::DataService::DEBUG;
+}
+
+
+# _process_record ( record, steps )
 # 
 # Process the specified record using the specified steps.
 
-sub process_record {
+sub _process_record {
     
     my ($request, $record, $steps) = @_;
     my $ds = $request->{ds};
@@ -338,7 +507,7 @@ sub process_record {
 
 sub result_limit {
     
-    return $_[0]->{result_limit} ne 'all' && $_[0]->{result_limit};
+    return defined $_[0]->{result_limit} && $_[0]->{result_limit} ne 'all' && $_[0]->{result_limit};
 }
 
 
@@ -392,6 +561,42 @@ sub sql_limit_clause {
 }
 
 
+# require_preprocess ( arg )
+# 
+# If the argument is true, then the result set will be processed before
+# output. This will mean that the entire result set will be held in the memory
+# of the dataservice process before being sent to the client, no matter how
+# big it is.
+# 
+# If the argument is '2', then this will only be done if row counts were
+# requested and not otherwise.
+
+sub require_preprocess {
+    
+    my ($request, $arg) = @_;
+    
+    croak "you must provide a defined argument, either 0, 1, or 2"
+	unless defined $arg && ($arg eq '0' || $arg eq '1' || $arg eq '2');
+    
+    if ( $arg eq '2' )
+    {
+	$request->{process_before_count} = 1;
+	$request->{preprocess} = 0;
+    }
+    
+    elsif ( $arg eq '1' )
+    {
+	$request->{preprocess} = 1;
+    }
+    
+    elsif ( $arg eq '0' )
+    {
+	$request->{process_before_count} = 0;
+	$request->{preprocess} = 0;
+    }
+}
+
+
 # sql_count_clause ( )
 # 
 # Return a string that can be added to an SQL statement to generate a result
@@ -464,6 +669,42 @@ sub warnings {
 }
 
 
+sub add_caution {
+
+    my ($self, $error_msg) = @_;
+    
+    $self->{cautions} = [] unless ref $self->{cautions} eq 'ARRAY';
+    push @{$self->{cautions}}, $error_msg;
+}
+
+
+sub cautions {
+    
+    my ($self) = @_;
+    
+    return @{$self->{cautions}} if ref $self->{cautions} eq 'ARRAY';
+    return;
+}
+
+
+sub add_error {
+    
+    my ($self, $error_msg) = @_;
+    
+    $self->{errors} = [] unless ref $self->{errors} eq 'ARRAY';
+    push @{$self->{errors}}, $error_msg;
+}
+
+
+sub errors {
+
+    my ($self) = @_;
+    
+    return @{$self->{errors}} if ref $self->{errors} eq 'ARRAY';
+    return;
+}
+
+
 # display_header
 # 
 # Return true if we should display optional header material, false
@@ -541,6 +782,17 @@ sub params_for_display {
 	
 	my @values = $request->clean_param_list($p);
 	
+	# Go through the values; if any one is an object with a 'regenerate'
+	# method, then call it.
+	
+	foreach my $v (@values)
+	{
+	    if ( ref $v && $v->can('regenerate' ) )
+	    {
+		$v = $v->regenerate;
+	    }
+	}
+	
 	push @display, $p, join(q{,}, @values);
     }
     
@@ -608,7 +860,9 @@ sub result_counts {
 
 sub linebreak {
 
-    return $_[0]->{linebreak_cr} ? "\n" : "\r\n";
+    return $_[0]->{output_linebreak} eq 'cr' ? "\r"
+	 : $_[0]->{output_linebreak} eq 'lf' ? "\n"
+					     : "\r\n";
 }
 
 
@@ -840,4 +1094,17 @@ sub clear_result {
 }
 
 
+# skip_output_record
+#
+# This method should only be called from a before_record_hook. It directs that the record about to
+# be output should be skipped.
+
+sub skip_output_record {
+
+    my ($request) = @_;
+    
+    $request->{_skip_record} = 1;
+}
+
+
 1;
