@@ -23,7 +23,6 @@ our (%NODE_DEF) = ( path => 'ignore',
 		    list => 'single',
 		    title => 'single',
 		    usage => 'single',
-		    collapse_tree => 'single',
 		    file_dir => 'single',
 		    file_path => 'single',
 		    role => 'single',
@@ -45,11 +44,14 @@ our (%NODE_DEF) = ( path => 'ignore',
 		    default_linebreak => 'single',
 		    default_save_filename => 'single',
 		    stream_theshold => 'single',
-		    init_operation_hook => 'hook',
-		    post_params_hook => 'hook',
-		    post_configure_hook => 'hook',
-		    post_process_hook => 'hook',
-		    output_record_hook => 'hook',
+		    before_execute_hook => 'hook',
+		    before_config_hook => 'hook',
+		    before_setup_hook => 'hook',
+		    before_operation_hook => 'hook',
+		    before_output_hook => 'hook',
+		    before_record_hook => 'hook',
+		    after_serialize_hook => 'hook',
+		    post_configure_hook => 'hook',	# deprecated
 		    use_cache => 'single',
 		    allow_method => 'set',
 		    allow_format => 'set',
@@ -685,7 +687,7 @@ sub node_attr {
 	no warnings;
 	if ( exists $ds->{attr_cache}{$path}{$key} )
 	{
-	    $ds->{attr_cache}{$path}{$key};
+	    return $ds->{attr_cache}{$path}{$key};
 	    #return ref $ds->{attr_cache}{$path}{$key} eq 'ARRAY' ?
 	    #	@{$ds->{attr_cache}{$path}{$key}} : $ds->{attr_cache}{$path}{$key};
 	}
@@ -844,6 +846,21 @@ sub _lookup_node_attr {
 	    push @$new_value, $v if defined $v && $v ne '';
 	}
     }
+
+    # If the attribute type is 'hook', then add the new value to the end of the previous list.
+
+    elsif ( $NODE_DEF{$key} eq 'hook' )
+    {
+	if ( ref $inherited_value eq 'ARRAY' && @$inherited_value )
+	{
+	    $new_value = [ @$inherited_value, @{$ds->{node_attrs}{$path}{$key}} ];
+	}
+	
+	else
+	{
+	    $new_value = $ds->{node_attrs}{$path}{$key};
+	}
+    }
     
     # Otherwise, the new value simply overrides any inherited value.  This code
     # path is only here in case path_compose is set mistakenly for some attribute
@@ -924,6 +941,11 @@ sub add_node_doc {
 	}
     }
     
+    # Change any initial > or >> into a blank line, to indicate a new
+    # paragraph.
+    
+    $doc =~ s{^>>?}{\n}xs;
+    
     # Now add the documentation string.
     
     $node->{doc_string} = '' unless defined $node->{doc_string};
