@@ -1,7 +1,8 @@
 # 
-# DataService::Output
-# 
+# Web::DataService::Output
 # 
+# This module provides a role that is used by 'Web::DataService'.  It implements
+# routines for configuring and generating data service output.
 # 
 # Author: Michael McClennen
 
@@ -44,6 +45,20 @@ sub define_block {
 	croak "define_block: invalid block name '$name'";
     }
     
+    # Make sure the block name is unique.
+    
+    if ( $ds->{block}{$name} )
+    {
+	my $location = $ds->{block_loc}{$name};
+	croak "define_block: '$name' was already defined at $location\n";
+    }
+    
+    else
+    {
+	my ($package, $filename, $line) = caller;
+	$ds->{block_loc}{$name} = "$filename at line $line";
+    }
+    
     # Create a new block object.
     
     my $block = { name => $name,
@@ -82,17 +97,14 @@ sub define_block {
 	
 	$ds->add_doc($block, $item) if $type eq 'output';
 	
-	# Add the record to the appropriate list.
+	# Add the record to the appropriate list(s).
 	
 	if ( $type eq 'include' )
 	{
 	    push @{$ds->{block}{$name}{include_list}}, $item;
 	}
 	
-	else
-	{
-	    push @{$ds->{block}{$name}{output_list}}, $item;
-	}
+	push @{$ds->{block}{$name}{output_list}}, $item;
     }
     
     $ds->process_doc($block);
@@ -104,6 +116,7 @@ our %OUTPUT_DEF = (output => 'type',
 		   select => 'type',
 		   filter => 'type',
 		   include => 'type',
+		   check => 'type',
 		   if_block => 'set',
 		   not_block => 'set',
 		   if_vocab => 'set',
@@ -131,19 +144,20 @@ our %OUTPUT_DEF = (output => 'type',
 		   split => 'regexp',
 		   join => 'single',
 		   tables => 'set',
+		   disabled => 'single',
 		   doc_string => 'single');
 
-our %SELECT_KEY = (select => 1, tables => 1);
+our %SELECT_KEY = (select => 1, tables => 1, if_block => 1);
 
 our %FIELD_KEY = (dedup => 1, name => 1, value => 1, always => 1, sub_record => 1, if_field => 1, 
 		  not_field => 1, if_block => 1, not_block => 1, if_format => 1, not_format => 1,
 		  if_vocab => 1, not_vocab => 1,
-		  text_join => 1, xml_join => 1, doc_string => 1, show_as_list => 1, undocumented => 1);
+		  text_join => 1, xml_join => 1, doc_string => 1, show_as_list => 1, disabled => 1, undocumented => 1);
 
-our %PROC_KEY = (set => 1, append => 1, from => 1, from_each => 1, 
+our %PROC_KEY = (set => 1, check => 1, append => 1, from => 1, from_each => 1, 
 		 if_vocab => 1, not_vocab => 1, if_block => 1, not_block => 1,
 	         if_format => 1, not_format => 1, if_field => 1, not_field => 1,
-		 code => 1, lookup => 1, split => 1, join => 1, default => 1);
+		 code => 1, lookup => 1, split => 1, join => 1, default => 1, disabled => 1);
 
 sub check_output_record {
     
@@ -302,7 +316,7 @@ sub _setup_output {
 	    
 	    if ( $block && ref $ds->{block}{$block} eq 'Web::DataService::Block' )
 	    {
-		$request->{block_hash}{$block} = 1;
+		$request->{block_hash}{$block} = $key;
 		push @blocks, $block;
 	    }
 	    
@@ -333,13 +347,17 @@ sub _setup_output {
     # are included.
     
     my %uniq_block;
+    my @include_scan = @blocks;
+    my $bound = 0;
     
  INCLUDE_BLOCK:
-    foreach my $block (@blocks)
+    while ( my $block = shift @include_scan )
     {
-	# Make sure that each block is checked only once.
+	# Make sure that each block is checked only once, and add a bounds
+	# check to prevent a runaway loop.
 	
 	next if $uniq_block{$block}; $uniq_block{$block} = 1;
+	next if ++$bound > 999;
 	
 	my $include_list = $ds->{block}{$block}{include_list};
 	next unless ref $include_list eq 'ARRAY';
@@ -397,26 +415,21 @@ sub _setup_output {
 	    
 	    $request->{block_keys}{$include_key} = 1 if $include_key;
 	    $request->{block_hash}{$include_block} = 1 if $include_block;
-	    push @blocks, $include_block if $include_block;
+	    push @include_scan, $include_block if $include_block;
 	}
     }
     
     # Now run through all of the blocks we have identified and collect up the
     # various kinds of records they contain.
     
-    %uniq_block = ();
+    %uniq_block = ();	# $$$$
     
  BLOCK:
     foreach my $block (@blocks)
     {
-	# Make sure that each block is only processed once, even if it is
-	# listed more than once.
-	
-	next if $uniq_block{$block}; $uniq_block{$block} = 1;
-	
 	# Add this block to the output configuration.
 	
-	$ds->add_output_block($request, $block);
+	$ds->add_output_block($request, \%uniq_block, $block);
     }
     
     my $a = 1;	# We can stop here when debugging
@@ -430,7 +443,12 @@ sub _setup_output {
 
 sub add_output_block {
 
-    my ($ds, $request, $block_name) = @_;
+    my ($ds, $request, $uniq_block, $block_name) = @_;
+    
+    # Make sure that each block is only processed once, even if it is
+    # listed more than once.
+    
+    return if $uniq_block->{$block_name}; $uniq_block->{$block_name} = 1;
     
     # Generate a warning if the specified block does not exist, but do
     # not abort the request.
@@ -529,9 +547,9 @@ sub add_output_block {
 	    $request->{filter_hash}{$r->{filter}} = $r->{value};
 	}
 	
-	# If the record type is 'set', add a record to the process list.
+	# If the record type is 'set' or 'check', add a record to the process list.
 	
-	elsif ( defined $r->{set} )
+	elsif ( defined $r->{set} || defined $r->{check} )
 	{
 	    my $proc = { set => $r->{set} };
 	    
@@ -549,6 +567,29 @@ sub add_output_block {
 	    }
 	    
 	    push @{$request->{proc_list}}, $proc;
+	    
+	    # If this is a 'check' rule, then complain if the values don't
+	    # make sense.  Also note that we will have to process the result
+	    # set in its entirety if we need to compute the size.
+	    
+	    if ( defined $r->{check} )
+	    {
+		$request->{process_before_count} = 1;
+		
+		my $check_value = $r->{check};
+		
+		if ( $check_value eq '*' || $check_value eq '' )
+		{
+		    croak "the value of 'code' must be a code ref"
+			unless ref $r->{code} eq 'CODE';
+		}
+		
+		elsif ( defined $r->{lookup} )
+		{
+		    croak "the value of 'lookup' must be a hash ref"
+			unless ref $r->{lookup} eq 'HASH';
+		}
+	    }
 	}
 	
 	# If the record type is 'output', add a record to the field list.
@@ -557,6 +598,8 @@ sub add_output_block {
 	
 	elsif ( defined $r->{output} )
 	{
+	    croak "the value of 'output' must be non-empty" unless $r->{output} ne '';
+	    
 	    next RECORD if $require_vocab and not exists $r->{"${vocab}_name"};
 	    
 	    my $field = { field => $r->{output}, name => $r->{output} };
@@ -582,12 +625,14 @@ sub add_output_block {
 		
 		elsif ( $key eq 'data_type' )
 		{
-		    my $type_value = $r->{$key};
-		    croak "unknown value '$r->{$key}' for data_type: must be one of 'int', 'pos', 'dec'"
+		    my $type_value = $r->{data_type};
+		    croak "unknown value '$r->{data_type}' for data_type: must be one of 'int', 'pos', 'dec', 'str'"
 			unless lc $type_value eq 'int' || lc $type_value eq 'pos' ||
-			    lc $type_value eq 'dec';
+			    lc $type_value eq 'dec' || lc $type_value eq 'str';
 		    
-		    push @{$request->{proc_list}}, { check => $r->{output}, type => $r->{$key} };
+		    $field->{data_type} = $r->{data_type};
+		    push @{$request->{proc_list}}, { check_field => $r->{output}, data_type => $r->{data_type} }
+			unless $r->{data_type} eq 'str';
 		}
 		
 		elsif ( $key ne 'output' )
@@ -612,8 +657,8 @@ sub add_output_block {
 	    # comes up again.
 	    
 	    my $include_block = $r->{include_block};
-	    next RECORD if $request->{block_hash}{$include_block};
-	    $request->{block_hash}{$include_block} = 1;
+	    next RECORD if $uniq_block->{$include_block};
+	    $uniq_block->{$include_block} = 1;
 	    
 	    # Get the list of block records, or add a warning if no block
 	    # was defined under that name.
@@ -1258,6 +1303,7 @@ sub document_response {
     # specified.
     
     my $optional_output = $ds->node_attr($path, 'optional_output');
+    my $reverse_map;
     
     if ( $optional_output && ref $ds->{set}{$optional_output} eq 'Web::DataService::Set' )
     {
@@ -1272,6 +1318,8 @@ sub document_response {
 	    next VALUE if $output_map->{value}{$label}{disabled} || 
 		$output_map->{value}{$label}{undocumented};
 	    
+	    $reverse_map->{$block_name} = $label;
+	    
 	    if ( ref $ds->{block}{$block_name} eq 'Web::DataService::Block' )
 	    {
 		push @blocks, $block_name;
@@ -1323,7 +1371,7 @@ sub document_response {
     
     if ( $field_count > 1 )
     {
-	$doc_string .= "=for wds_table_header Field name*/$field_count | Block!anchor(block:) | Description\n\n";
+	$doc_string .= "=for wds_table_header Field name*/$field_count | Block | Description\n\n";
 	$doc_string .= "=over 4\n\n";
 	$doc_string .= "=item $field_string\n\n";
     }
@@ -1355,7 +1403,7 @@ sub document_response {
 	foreach my $r (@$output_list)
 	{
 	    next unless defined $r->{output};
-	    $doc_string .= $ds->document_field($block_label, \@vocab_list, $r)
+	    $doc_string .= $ds->document_field($block_label, \@vocab_list, $r, $reverse_map)
 		unless $r->{undocumented};
 	}
     }
@@ -1405,7 +1453,7 @@ sub document_summary {
     
     if ( $field_count > 1 )
     {
-	$doc_string .= "=for wds_table_header Field name*/$field_count | Block!anchor(block:) | Description\n\n";
+	$doc_string .= "=for wds_table_header Field name*/$field_count | Block | Description\n\n";
 	$doc_string .= "=over 4\n\n";
 	$doc_string .= "=item $field_string\n\n";
     }
@@ -1424,7 +1472,7 @@ sub document_summary {
     foreach my $r (@$output_list)
     {
 	next unless defined $r->{output};
-	$doc_string .= $ds->document_field('summary', \@vocab_list, $r)
+	$doc_string .= $ds->document_field('summary', \@vocab_list, $r, {})
 	    unless $r->{undocumented};
     }
     
@@ -1436,7 +1484,7 @@ sub document_summary {
 
 sub document_field {
     
-    my ($ds, $block_key, $vocab_list, $r) = @_;
+    my ($ds, $block_key, $vocab_list, $r, $rev_map) = @_;
     
     my @names;
     
@@ -1447,6 +1495,8 @@ sub document_field {
 	      : $ds->{vocab}{$v}{use_field_names} ? $r->{output}
 	      :					      '';
 	
+	$n ||= 'I<n/a>';
+	
 	push @names, $n
     }
     
@@ -1458,11 +1508,11 @@ sub document_field {
     {
 	if ( ref $r->{if_block} eq 'ARRAY' )
 	{
-	    $block_key = join(', ', @{$r->{if_block}});
+	    $block_key = join(', ', map { $rev_map->{$_} // $_ } @{$r->{if_block}});
 	}
 	else
 	{
-	    $block_key = $r->{if_block};
+	    $block_key = $rev_map->{$r->{if_block}} // $r->{if_block};
 	}
     }
     
@@ -1475,7 +1525,8 @@ sub document_field {
 # process_record ( request, record, steps )
 # 
 # Execute any per-record processing steps that have been defined for this
-# record. 
+# record. Return true if the record is to be included in the result, false
+# otherwise.
 
 sub process_record {
     
@@ -1483,22 +1534,90 @@ sub process_record {
     
     # If there are no processing steps to do, return immediately.
     
-    return unless ref $steps eq 'ARRAY' and @$steps;
+    return 1 unless ref $steps eq 'ARRAY' and @$steps;
     
     # Otherwise go through the steps one by one.
     
     foreach my $p ( @$steps )
     {
-	# If this step is a 'check' step, then do the check.
+	# Skip this processing step based on a conditional field value, if one
+	# is defined.
 	
-	if ( exists $p->{check} )
+	if ( my $cond_field = $p->{if_field} )
 	{
-	    $ds->check_field_type($record, $p->{check}, $p->{type}, $p->{subst});
+	    next unless defined $record->{$cond_field};
+	    next if ref $record->{$cond_field} eq 'ARRAY' && @{$record->{$cond_field}} == 0;
+	}
+	
+	elsif ( $cond_field = $p->{not_field} )
+	{
+	    next if defined $record->{$cond_field} && ref $record->{$cond_field} ne 'ARRAY';
+	    next if ref $record->{$cond_field} eq 'ARRAY' && @{$record->{$cond_field}} > 0;
+	}
+	
+	# If this step is a 'check_field' step, then do the check.
+	
+	if ( defined $p->{check_field} )
+	{
+	    $ds->check_field_type($record, $p->{check_field}, $p->{data_type}, $p->{subst});
 	    next;
 	}
 	
-	# Figure out which field (if any) we are affecting.  A value of '*'
-	# means to use the entire record (only relevant with 'code').
+	# If this step is a 'check' step (i.e. check the entire record) then
+	# do the check. If it fails, we return false.
+	
+	elsif ( defined $p->{check} )
+	{
+	    my $check_value = $p->{check};
+	    
+	    # If the value is '*' or the empty string, then we must have a
+	    # code reference to call.
+	    
+	    if ( $check_value eq '' || $check_value eq '*' )
+	    {
+		return $p->{code}($request, $record);
+	    }
+	    
+	    # Otherwise, if the value is '1' or '0' then return that.  The
+	    # former will cause the record to be included in the result, the
+	    # latter will cause it to be skipped.  This is mainly useful in
+	    # conjunction with 'if_field' or 'not_field'.
+	    
+	    elsif ( $check_value eq '1' || $check_value eq '0' )
+	    {
+		return $check_value;
+	    }
+	    
+	    # Otherwise, we assume that we have been given a field name and
+	    # either call a code reference or do a hash-table lookup.
+	    
+	    elsif ( defined $p->{code} )
+	    {
+		my $value = $record->{$check_value};
+		return $p->{code}($request, $value);
+	    }
+	    
+	    elsif ( defined $p->{lookup} )
+	    {
+		my $value = $record->{$check_value};
+		return $p->{lookup}{$value} // $p->{default};
+	    }
+	    
+	    # Otherwise, we just return the value of the specified field. The
+	    # record will be included if this value is true, and skipped if
+	    # false.
+	    
+	    else
+	    {
+		return $record->{$check_value};
+	    }
+	    
+	    next;
+	}
+	
+	# If we get here, the current rule must be a 'set'.  Figure out which
+	# field (if any) we are affecting.  A value of '*' means to use the
+	# entire record (only relevant with 'code').
 	
 	my $set_field = $p->{set};
 	
@@ -1518,21 +1637,6 @@ sub process_record {
 	    next if ref $record->{$source_field} eq 'ARRAY' && @{$record->{$source_field}} == 0;
 	}
 	
-	# Skip this processing step based on a conditional field value, if one
-	# is defined.
-	
-	if ( my $cond_field = $p->{if_field} )
-	{
-	    next unless defined $record->{$cond_field};
-	    next if ref $record->{$cond_field} eq 'ARRAY' && @{$record->{$cond_field}} == 0;
-	}
-	
-	elsif ( $cond_field = $p->{not_field} )
-	{
-	    next if defined $record->{$cond_field} && ref $record->{$cond_field} ne 'ARRAY';
-	    next if ref $record->{$cond_field} eq 'ARRAY' && @{$record->{$cond_field}} > 0;
-	}
-	
 	# Now generate a list of result values, according to the attributes of this
 	# processing step.
 	
@@ -1687,7 +1791,9 @@ sub process_record {
 		$record->{$set_field} = '';
 	    }
 	}
-    }    
+    }
+    
+    return 1;
 }
 
 
@@ -1714,7 +1820,7 @@ sub check_field_type {
     
     elsif ( $type eq 'dec' )
     {
-	return if $record->{$field} =~ qr< ^ -? (?: [1-9][0-9]* (?: \. [0-9]* )? | [0]? \. [0-9]+ | [0] \. ) $ >x;
+	return if $record->{$field} =~ qr< ^ -? (?: [1-9][0-9]* (?: \. [0-9]* )? | [0]? \. [0-9]+ | [0] \.? ) $ >x;
     }
     
     elsif ( $type eq 'sci' )
@@ -1762,6 +1868,9 @@ sub _generate_single_result {
     die "could not generate a result in format '$format': no implementing module was found"
 	unless $format_class;
     
+    my $path = $request->node_path;
+    my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
+    
     # Set the result count to 1, in case the client asked for it.
     
     $request->{result_count} = 1;
@@ -1772,42 +1881,60 @@ sub _generate_single_result {
     my $proc_list = $request->{proc_list};
     my $field_list = $request->{field_list};
     
-    # Generate the initial part of the output, before the first record.
+    # Make sure we have at least one field to output.
     
-    my $output = $format_class->emit_header($request, $field_list);
+    unless ( ref $field_list && @$field_list )
+    {
+	$request->add_warning("No output fields were defined for this request.");
+    }
     
     # If there are any processing steps to do, then do them.
     
     $ds->process_record($request, $request->{main_record}, $proc_list);
     
-    # If there is an output_record_hook defined for this path, call it now.
-    # If it returns false, do not output the record.
+    # If there is a before_record_hook defined for this path, call it now. For a single result,
+    # calls to 'skip_output_record' are not allowed.
     
-    if ( $request->{output_record_hook} )
+    if ( $ds->{hook_enabled}{before_record_hook} )
     {
-	$ds->call_hook($request->{output_record_hook}, $request, $request->{main_record})
-	    or return;
+	$ds->_call_hooks($path, 'before_record_hook', $request, $request->{main_record})
     }
     
+    # Generate the initial part of the output, before the first record.
+    
+    my $header = $format_class->emit_header($request, $field_list);
+    
     # Generate the output corresponding to our single record.
     
-    $output .= $format_class->emit_record($request, $request->{main_record}, $field_list);
+    my $record = $format_class->emit_record($request, $request->{main_record}, $field_list);
     
     # Generate the final part of the output, after the last record.
     
-    $output .= $format_class->emit_footer($request, $field_list);
+    my $footer = $format_class->emit_footer($request, $field_list);
+
+    # If an after_serialize_hook is defined for this path, call it.
     
-    return $output;
+    if ( $serial_hook )
+    {
+	my $rs = '';
+	
+	$ds->_call_hook_list($serial_hook, $request, 'header', \$header);
+	$ds->_call_hook_list($serial_hook, $request, 'record', \$rs, \$record);
+	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+    }
+    
+    return $header . $record . $footer;
 }
 
 
 # _generate_compound_result ( request )
 # 
-# This function is called after an operation is executed.  It serializes each
-# result record according to the specified output format and returns the
-# resulting string.  If $streaming_threshold is specified, and if the size of
-# the output exceeds this threshold, this routine then sets up to stream the
-# rest of the output.
+# This function is called after an operation is executed and returns a result
+# set, provided that the entire result set does not need to be processed
+# before output.  It serializes each result record according to the specified output
+# format and returns the resulting string.  If $streaming_threshold is
+# specified, and if the size of the output exceeds this threshold, this
+# routine then sets up to stream the rest of the output.
 
 sub _generate_compound_result {
 
@@ -1821,6 +1948,9 @@ sub _generate_compound_result {
     die "could not generate a result in format '$format': no implementing module was found"
 	unless $format_class;
     
+    my $path = $request->node_path;
+    my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
+    
     # Get the lists that specify how to process each record and which fields
     # to output.
     
@@ -1836,6 +1966,11 @@ sub _generate_compound_result {
     
     my $output = $format_class->emit_header($request, $field_list);
     
+    if ( $serial_hook )
+    {
+	$ds->_call_hook_list($serial_hook, $request, 'header', \$output);	
+    }
+    
     # A record separator is emitted before every record except the first.  If
     # this format class does not define a record separator, use the empty
     # string.
@@ -1847,47 +1982,68 @@ sub _generate_compound_result {
     
     $request->{actual_count} = 0;
     
-    # If an offset was specified and the result method didn't handle this
-    # itself, then skip the specified number of records.
+    # If we have a result limit of 0, just output the header and footer and
+    # don't bother about the records.
     
-    if ( defined $request->{result_offset} && $request->{result_offset} > 0
+    if ( defined $request->{result_limit} && $request->{result_limit} eq '0' )
+    {
+	$request->{limit_zero} = 1;
+    }
+    
+    # Otherwise, if an offset was specified and the result method didn't
+    # handle this itself, then skip the specified number of records.
+    
+    elsif ( defined $request->{result_offset} && $request->{result_offset} > 0
 	 && ! $request->{offset_handled} )
     {
-	$ds->_next_record($request) foreach 1..$request->{result_offset};
+	foreach (1..$request->{result_offset})
+	{
+	    $ds->_next_record($request) or last;
+	}
     }
     
     # Now fetch and process each output record in turn.  If output streaming is
     # available and our total output size exceeds the threshold, switch over
     # to streaming.
     
+ RECORD:
     while ( my $record = $ds->_next_record($request) )
     {
 	# If there are any processing steps to do, then process this record.
 	
 	$ds->process_record($request, $record, $proc_list);
 	
-	# If there is an output_record_hook defined for this path, call it now.
-	# If it returns false, do not output the record.
+	# If there is a before_record_hook defined for this path, call it now.
+	# If it calls 'skip_output_record', then do not output this record.
 	
-	if ( $request->{output_record_hook} )
+	if ( $ds->{hook_enabled}{before_record_hook} )
 	{
-	    $ds->call_hook($request->{output_record_hook}, $request, $request->{main_record})
-		or return;
+	    delete $request->{_skip_record};
+	    $ds->_call_hooks('before_record_hook', $request, $record);
+	    next RECORD if $request->{_skip_record};
 	}
 	
 	# Generate the output for this record, preceded by a record separator if
 	# it is not the first record.
 	
-	$output .= $request->{rs} if $emit_rs; $emit_rs = 1;
+	my $outrs = $emit_rs ? $request->{rs} : ''; $emit_rs = 1;
+	my $outrec = $format_class->emit_record($request, $record, $field_list);
 	
-	$output .= $format_class->emit_record($request, $record, $field_list);
+	if ( $serial_hook )
+	{
+	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	}
+	
+	$output .= $outrs . $outrec;
 	
 	# Keep count of the output records, and stop if we have exceeded the
 	# limit.
 	
+	$request->{actual_count}++;
+	
 	if ( defined $request->{result_limit} && $request->{result_limit} ne 'all' )
 	{
-	    last if ++$request->{actual_count} >= $request->{result_limit};
+	    last if $request->{actual_count} >= $request->{result_limit};
 	}
 	
 	# If streaming is a possibility, check whether we have passed the
@@ -1906,9 +2062,31 @@ sub _generate_compound_result {
     # If we get here, then we did not initiate streaming.  So add the
     # footer and return the output data.
     
+    # If we didn't output any records, give the formatter a chance to indicate
+    # this. 
+    
+    unless ( $request->{actual_count} )
+    {
+	my $empty = $format_class->emit_empty($request);
+	
+	if ( $serial_hook )
+	{
+	    $ds->_call_hook_list($serial_hook, $request, 'empty', \$empty);
+	}
+
+	$output .= $empty;
+    }
+    
     # Generate the final part of the output, after the last record.
     
-    $output .= $format_class->emit_footer($request, $field_list);
+    my $footer = $format_class->emit_footer($request, $field_list);
+
+    if ( $serial_hook )
+    {
+	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+    }
+    
+    $output .= $footer;
     
     # Determine if we need to encode the output into the proper character set.
     # Usually Dancer does this for us, but only if it recognizes the content
@@ -1928,42 +2106,198 @@ sub _generate_compound_result {
     return $must_encode ? encode($output_charset, $output) : $output;
 }
 
-    # If the flag 'process_resultset' is set, then we need to fetch and
-    # process the entire result set before generating output.  Obviously,
-    # streaming is not a possibility in this case.
+
+# _generate_processed_result ( request )
+# 
+# This function is called if the result set needs to be processed in its
+# entirety before being output.  It processes the entire result set and
+# collects a list of processed records, and then serializes each result record
+# according to the specified output format.  If $streaming_threshold is
+# specified, and if the size of the output exceeds this threshold, this
+# routine then sets up to stream the rest of the output.
+
+sub _generate_processed_result {
+
+    my ($ds, $request, $streaming_threshold) = @_;
     
-    # if ( $ds->{process_resultset} )
-    # {
-    # 	my @rows;
+    # Determine the output format and figure out which class implements it.
+    
+    my $format = $request->output_format;
+    my $format_class = $ds->{format}{$format}{package};
+    
+    die "could not generate a result in format '$format': no implementing module was found"
+	unless $format_class;
+    
+    $ds->debug_line("Processing result set before output.");
+    
+    my $path = $request->node_path;
+    my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
+    
+    # Get the lists that specify how to process each record and which fields
+    # to output.
+    
+    my $proc_list = $request->{proc_list};
+    my $field_list = $request->{field_list};
+    
+    # Now fetch and process each output record in turn.  Collect up all of the
+    # records that pass the processing phase in a list.
+    
+    my @results;
+    
+ RECORD:
+    while ( my $record = $ds->_next_record($request) )
+    {
+	# If there are any processing steps to do, then process this record.
+	# If the return value is not true, skip the record.
 	
-    # 	if ( $ds->{main_sth} )
-    # 	{
-    # 	    while ( $record = $ds->{main_sth}->fetchrow_hashref )
-    # 	    {
-    # 		push @rows, $record;
-    # 	    }
-    # 	}
+	$ds->process_record($request, $record, $proc_list) or next RECORD;
 	
-    # 	else
-    # 	{
-    # 	    @rows = @{$ds->{main_result}}
-    # 	}
+	# If there is a before_record_hook defined for this path, call it now.
+	# If it calls 'skip_output_record', then do not output this record.
 	
-    # 	my $newrows = $ds->{process_resultset}(\@rows);
+	if ( $ds->{hook_enabled}{before_record_hook} )
+	{
+	    delete $request->{_skip_record};
+	    $ds->_call_hooks('before_record_hook', $request, $record);
+	    next RECORD if $request->{_skip_record};
+	}
 	
-    # 	if ( ref $newrows eq 'ARRAY' )
-    # 	{
-    # 	    foreach my $record (@$newrows)
-    # 	    {
-    # 		$ds->processRecord($record, $ds->{proc_list});
-    # 		my $record_output = $ds->emitRecord($record, is_first => $first_row);
-    # 		$output .= $record_output;
-		
-    # 		$first_row = 0;
-    # 		$ds->{row_count}++;
-    # 	    }
-    # 	}
-    # }
+	# Add the record to the list.
+	
+	push @results, $record;
+    }
+    
+    # We now know the result count.
+    
+    $request->{result_count} = scalar(@results);
+    
+    # At this point, we can generate the output.  We start with the header.
+    
+    my $output = $format_class->emit_header($request, $field_list);
+    
+    if ( $serial_hook )
+    {
+	$ds->_call_hook_list($serial_hook, $request, 'header', \$output);	
+    }
+    
+    # A record separator is emitted before every record except the first.  If
+    # this format class does not define a record separator, use the empty
+    # string.
+    
+    $request->{rs} = $format_class->can('emit_separator') ?
+	$format_class->emit_separator($request) : '';
+    
+    my $emit_rs = 0;
+    
+    $request->{actual_count} = 0;
+    
+    # If an offset was specified and the result method didn't handle this
+    # itself, then skip the specified number of records.
+    
+    if ( defined $request->{result_offset} && $request->{result_offset} > 0
+	 && ! $request->{offset_handled} )
+    {
+	splice(@results, 0, $request->{result_offset});
+    }
+    
+    # If the result limit is zero, we can ignore all records.
+    
+    if ( defined $request->{result_limit} && $request->{result_limit} eq '0' )
+    {
+	@results = ();	
+    }
+    
+    # Otherwise iterate over all of the remaining records.
+    
+ OUTPUT:
+    while ( @results )
+    {
+	my $record = shift @results;
+	
+	# Generate the output for this record, preceded by a record separator if
+	# it is not the first record.
+	
+	my $outrs = $emit_rs ? $request->{rs} : ''; $emit_rs = 1;
+	my $outrec = $format_class->emit_record($request, $record, $field_list);
+	
+	if ( $serial_hook )
+	{
+	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	}
+	
+	$output .= $outrs . $outrec;
+	
+	# Keep count of the output records, and stop if we have exceeded the
+	# limit.
+	
+	$request->{actual_count}++;
+	
+	if ( defined $request->{result_limit} && $request->{result_limit} ne 'all' )
+	{
+	    last if $request->{actual_count} >= $request->{result_limit};
+	}
+	
+	# If streaming is a possibility, check whether we have passed the
+	# threshold for result size.  If so, then we need to immediately
+	# stash the output generated so far and call stream_data.  Doing that
+	# will cause the current function to be aborted, followed by an
+	# automatic call to &stream_result (defined below).
+	
+	if ( defined $streaming_threshold && length($output) > $streaming_threshold )
+	{
+	    $request->{stashed_output} = $output;
+	    $request->{stashed_results} = \@results;
+	    $request->{processing_complete} = 1;
+	    Dancer::Plugin::StreamData::stream_data($request, &_stream_compound_result);
+	}
+    }
+    
+    # If we get here, then we did not initiate streaming.  So add the
+    # footer and return the output data.
+    
+    # If we didn't output any records, give the formatter a chance to indicate
+    # this. 
+    
+    unless ( $request->{actual_count} )
+    {
+	my $empty = $format_class->emit_empty($request);
+	
+	if ( $serial_hook )
+	{
+	    $ds->_call_hook_list($serial_hook, $request, 'empty', \$empty);
+	}
+
+	$output .= $empty;
+    }
+    
+    # Generate the final part of the output, after the last record.
+    
+    my $footer = $format_class->emit_footer($request, $field_list);
+    
+    if ( $serial_hook )
+    {
+	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+    }
+    
+    $output .= $footer;
+    
+    # Determine if we need to encode the output into the proper character set.
+    # Usually Dancer does this for us, but only if it recognizes the content
+    # type as text.  For these formats, the definition should set the
+    # attribute 'encode_as_text' to true.
+    
+    my $output_charset = $ds->{_config}{charset};
+    my $must_encode;
+    
+    if ( $output_charset 
+	 && $ds->{format}{$format}{encode_as_text}
+	 && ! $request->{content_type_is_text} )
+    {
+	$must_encode = 1;
+    }
+    
+    return $must_encode ? encode($output_charset, $output) : $output;
+}
 
 
 # _stream_compound_result ( )
@@ -1994,6 +2328,15 @@ sub _stream_compound_result {
     croak "could not generate a result in format '$format': no implementing class"
 	unless $format_class;
     
+    my $path = $request->node_path;
+    my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
+    
+    # Get the lists that specify how to process each record and which fields
+    # to output.
+    
+    my $proc_list = $request->{proc_list};
+    my $field_list = $request->{field_list};
+    
     # Determine the output character set, because we will need to encode text
     # responses in it.
     
@@ -2016,30 +2359,41 @@ sub _stream_compound_result {
     
     # Then process the remaining rows.
     
+  RECORD:
     while ( my $record = $ds->_next_record($request) )
     {
-	# If there are any processing steps to do, then process this record.
-	
-	$ds->process_record($request, $record, $request->{proc_list});
-	
-	# If there is an output_record_hook defined for this path, call it now.
-	# If it returns false, do not output the record.
-	
-	if ( $request->{output_record_hook} )
+	# If there are any processing steps to do, then process this record. But skip this if this
+	# subroutine was called from '_generate_processed_result'.
+
+	unless ( $request->{processing_complete} )
 	{
-	    $ds->call_hook($request->{output_record_hook}, $request, $request->{main_record})
-		or return;
+	    $ds->process_record($request, $record, $proc_list);
+	    
+	    # If there is a before_record_hook defined for this path, call it now.
+	    # If it calls 'skip_output_record', then do not output this record.
+	    
+	    if ( $ds->{hook_enabled}{before_record_hook} )
+	    {
+		delete $request->{_skip_record};
+		$ds->_call_hooks('before_record_hook', $request, $record);
+		next RECORD if $request->{_skip_record};
+	    }
 	}
 	
-	# Generate the output for this record, preceded by a record separator
-	# since we are always past the first record once we have switched over
-	# to streaming.
+	# Generate the output for this record, preceded by a record separator if
+	# it is not the first record.
 	
-	my $output = $request->{rs};
+	my $outrs = $request->{rs};
+	my $outrec = $format_class->emit_record($request, $record, $field_list);
 	
-	$output .= $format_class->emit_record($request, $record);
+	if ( $serial_hook )
+	{
+	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	}
 	
-	unless ( defined $output and $output ne '' )
+	my $output .= $outrs . $outrec;
+	
+	if ( ! defined $output or $output eq '' )
 	{
 	    # do nothing
 	}
@@ -2068,9 +2422,30 @@ sub _stream_compound_result {
     
     # Finally, send out the footer and then close the writer object.
     
-    my $footer = $format_class->emit_footer($request);
+    # Generate the final part of the output, after the last record.
+    
+    my $footer = $format_class->emit_footer($request, $field_list);
+    
+    if ( $serial_hook )
+    {
+	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+    }
+    
+    if ( ! defined $footer or $footer eq '' )
+    {
+	# do nothing
+    }
+    
+    elsif ( $output_charset && $format_is_text )
+    {
+	$writer->write( encode($output_charset, $footer) );
+    }
+    
+    else
+    {
+	$writer->write( $footer );
+    }
     
-    $writer->write( encode_utf8($footer) ) if defined $footer and $footer ne '';
     $writer->close();
 }
 
@@ -2086,15 +2461,22 @@ sub _next_record {
     
     my ($ds, $request) = @_;
     
-    # If the result limit is 0, return nothing.  This prevents any records
-    # from being returned.
+    # If the request has a zero limit, and no processing needs to be done on
+    # the result set, then no records need to be returned.
     
-    return if defined $request->{result_limit} && $request->{result_limit} eq '0';
+    return if $request->{limit_zero};
+    
+    # If we have a stashed result list, return the next item in it.
+    
+    if ( ref $request->{stashed_results} eq 'ARRAY' )
+    {
+	return shift @{$request->{stashed_results}};
+    }
     
     # If we have a 'main_result' array with something in it, return the next
     # item in it.
     
-    if ( ref $request->{main_result} eq 'ARRAY' and @{$request->{main_result}} )
+    elsif ( ref $request->{main_result} eq 'ARRAY' and @{$request->{main_result}} )
     {
 	return shift @{$request->{main_result}};
     }
@@ -2136,6 +2518,7 @@ sub _generate_empty_result {
     
     my $output = $format_class->emit_header($request);
     
+    $output .= $format_class->emit_empty($request);
     $output .= $format_class->emit_footer($request);
     
     return $output;
