@@ -140,6 +140,26 @@ sub get_request_path {
 }
 
 
+# get_http_method ( request )
+# 
+# Return the HTTP method associated with this request.
+
+sub get_http_method {
+
+    return Dancer::request->method;
+}
+
+
+# get_content_type ( request )
+#
+# Return the content type associated with this request.
+
+sub get_content_type {
+
+    return Dancer::request->content_type;
+}
+
+
 # get_params ( request )
 # 
 # Return the parameters for the current request.
@@ -166,6 +186,18 @@ sub get_param {
 }
 
 
+# get_request_body ( )
+# 
+# Return the body of the request, or the empty string if there wasn't one.
+
+sub get_request_body {
+
+    my ($plugin, $request) = @_;
+    
+    return Dancer::request->body();
+}
+
+
 # set_cors_header ( request, arg )
 # 
 # Set the CORS access control header according to the argument.
