@@ -83,13 +83,14 @@ sub emit_header {
     # Check if we have been asked to report the result count, and if it is
     # available.
     
+    $output .= '"elapsed_time":' . sprintf("%.3g", $request->{elapsed}) . ",\n";
+    
     if ( $request->display_counts )
     {
 	my $counts = $request->result_counts;
 	
-	$output .= '"elapsed_time":' . sprintf("%.3g", $request->{elapsed}) . ",\n";
-	$output .= '"records_found":' . json_clean($counts->{found}) . ",\n";
-	$output .= '"records_returned":' . json_clean($counts->{returned}) . ",\n";
+	$output .= '"records_found":' . json_clean($counts->{found} || '0') . ",\n";
+	$output .= '"records_returned":' . json_clean($counts->{returned} || '0') . ",\n";
 	$output .= '"record_offset":' . json_clean($counts->{offset}) . ",\n"
 	    if defined $counts->{offset} && $counts->{offset} > 0;
     }
@@ -135,6 +136,18 @@ sub emit_separator {
 }
 
 
+# emit_empty ( )
+# 
+# Return the string (if any) to output in lieu of an empty result set.
+
+sub emit_empty {
+    
+    my ($class, $request) = @_;
+    
+    return '';
+}
+
+
 # emit_footer ( )
 # 
 # Return the final text for a JSON result.
@@ -153,7 +166,7 @@ sub emit_footer {
 
 sub emit_error {
     
-    my ($class, $code, $errors, $warnings) = @_;
+    my ($class, $code, $errors, $warnings, $cautions) = @_;
     
     unless ( ref $errors eq 'ARRAY' )
     {
@@ -166,7 +179,8 @@ sub emit_error {
     }
     
     my $error = '"status_code": ' . $code;
-    $error .= ",\n" . json_list_value("errors", @$errors);
+    $error .= ",\n" . json_list_value("errors", @$errors) if ref $errors eq 'ARRAY' && @$errors;
+    $error .= ",\n" . json_list_value("cautions", @$cautions) if ref $cautions eq 'ARRAY' && @$cautions;
     $error .= ",\n" . json_list_value("warnings", @$warnings) if ref $warnings eq 'ARRAY' && @$warnings;
     
     return "{ $error }\n";
@@ -208,6 +222,7 @@ sub emit_object {
 	# Skip any field that is empty, unless 'always' or 'value' is set.
 	
 	my $field = $f->{field};
+	my $data_type = $f->{data_type};
 	
 	next unless $f->{always} or defined $f->{value} or 
 	    defined $record->{$field} and $record->{$field} ne '';
@@ -252,7 +267,7 @@ sub emit_object {
 	    
 	    if ( ref $value && reftype $value eq 'HASH' )
 	    {
-		$request->process_record($value, $proc_list) if $proc_list && @$proc_list;
+		$request->_process_record($value, $proc_list) if $proc_list && @$proc_list;
 		
 		if ( $output_list && @$output_list )
 		{
@@ -260,7 +275,7 @@ sub emit_object {
 		}
 		else
 		{
-		    $value = json_clean($value);
+		    $value = json_clean($value, $data_type);
 		}
 	    }
 	    
@@ -273,7 +288,7 @@ sub emit_object {
 		{
 		    foreach my $v ( @$value )
 		    {
-			$request->process_record($v, $proc_list) if $proc_list;
+			$request->_process_record($v, $proc_list) if $proc_list;
 		    }
 		}
 		
@@ -283,13 +298,13 @@ sub emit_object {
 		}
 		else
 		{
-		    $value = json_clean($value);
+		    $value = json_clean($value, $data_type);
 		}
 	    }
 	    
 	    else
 	    {
-		$value = json_clean($value);
+		$value = json_clean($value, $data_type);
 	    }
 	}
 	
@@ -311,7 +326,7 @@ sub emit_object {
 	
 	else
 	{
-	    $value = json_clean($value);
+	    $value = json_clean($value, $data_type);
 	}
 	
 	# Now, add the value to the growing output.  Add a comma before each
@@ -429,15 +444,18 @@ my (%ESCAPE) = ( '\\' => '\\\\', '"' => '\\"', "\t" => '\\t', "\n" => '\\n',
 
 sub json_clean {
     
-    my ($string) = @_;
+    my ($string, $data_type) = @_;
     
     # Return an empty string unless the value is defined.
     
     return '""' unless defined $string and $string ne '';
     
-    # Do a quick check for numbers.  If it matches, return the value as-is.
+    # Do a quick check for numbers.  If it matches, return the value as-is
+    # unless the data_type is 'str'.  In that case, the field value is
+    # intended to be a string so we should quote it even if it looks like a number.
     
-    return $string if $string =~ qr{ ^ -? (?: [1-9][0-9]* | 0 ) (?: \. [0-9]+ )? (?: [Ee] -? [0-9]+ )? $ }x;
+    return $string if $string =~ qr{ ^ -? (?: [1-9][0-9]* | 0 ) (?: \. [0-9]+ )? (?: [Ee] -? [0-9]+ )? $ }x
+	and not (defined $data_type && $data_type eq 'str');
     
     # Do another quick check for okay characters.  If there's nothing exotic,
     # just return the quoted value.
