@@ -86,7 +86,7 @@ sub emit_header {
 	$output .= $class->emit_line($request, "Elapsed Time", sprintf("%.3g", $request->{elapsed}));
 	$output .= $class->emit_line($request, "Records Found", $counts->{found});
 	$output .= $class->emit_line($request, "Records Returned", $counts->{returned});
-	$output .= $class->generate-line($request, "Record Offset", $counts->{offset})
+	$output .= $class->emit_line($request, "Record Offset", $counts->{offset})
 	    if defined $counts->{offset} && $counts->{offset} > 0;
     }
     
@@ -118,20 +118,13 @@ sub emit_header {
     # Now, if any output fields were specified for this request, list them in
     # a header line.
     
-    if ( ref $field_list eq 'ARRAY' )
+    if ( ref $field_list eq 'ARRAY' && @$field_list )
     {
 	my @fields = map { $_->{name} } @$field_list;
 	
 	$output .= $class->emit_line($request, @fields);
     }
     
-    # Otherwise, note that no fields are available.
-    
-    else
-    {
-	$output .= $class->emit_line($request, "THIS REQUEST DID NOT GENERATE ANY OUTPUT RECORDS");
-    }
-    
     # Return the text that we have generated.
     
     return $output;
@@ -155,6 +148,18 @@ sub generate_label {
 }
 
 
+# emit_empty ( )
+# 
+# Return the string (if any) to output in lieu of an empty result set.
+
+sub emit_empty {
+    
+    my ($class, $request) = @_;
+    
+    return $class->emit_line($request, "THIS REQUEST RETURNED NO RECORDS");
+}
+
+
 # emit_footer ( request )
 # 
 # None of the formats handled by this module involve any text after the last record
@@ -208,6 +213,12 @@ sub emit_record {
 	$v = '' if $f->{if_field} and not $record->{$f->{if_field}};
 	$v = '' if $f->{not_field} and $record->{$f->{not_field}};
 	
+	# Cancel out any field with a 'dedup' attribute if its value is the same
+	# as the value of the field indicated by the attribute.
+	
+	$v = '' if $f->{dedup} and defined $record->{$f->{field}} and defined $record->{$f->{dedup}}
+	    and $record->{$f->{field}} eq $record->{$f->{dedup}};
+	
 	# If the value is an array, join it into a string.  If no joining
 	# string was specified, use a comma.
 	
